@@ -22,9 +22,7 @@ import matplotlib.pyplot as plt
 from sklearn import linear_model
 from sklearn import datasets
 
-diabetes = datasets.load_diabetes()
-X = diabetes.data
-y = diabetes.target
+X, y = datasets.load_diabetes(return_X_y=True)
 
 print("Computing regularization path using the LARS ...")
 _, _, coefs = linear_model.lars_path(X, y, method='lasso', verbose=True)
