@@ -22,7 +22,7 @@ X = np.random.randn(300, 2)
 Y = np.logical_xor(X[:, 0] > 0, X[:, 1] > 0)
 
 # fit the model
-clf = svm.NuSVC()
+clf = svm.NuSVC(gamma='auto')
 clf.fit(X, Y)
 
 # plot the decision function for each datapoint on the grid
@@ -33,7 +33,7 @@ plt.imshow(Z, interpolation='nearest',
            extent=(xx.min(), xx.max(), yy.min(), yy.max()), aspect='auto',
            origin='lower', cmap=plt.cm.PuOr_r)
 contours = plt.contour(xx, yy, Z, levels=[0], linewidths=2,
-                       linetypes='--')
+                       linestyles='dashed')
 plt.scatter(X[:, 0], X[:, 1], s=30, c=Y, cmap=plt.cm.Paired,
             edgecolors='k')
 plt.xticks(())
