@@ -7,29 +7,34 @@
 #
 # License: BSD 3 clause
 
-from __future__ import division
 import warnings
+from inspect import signature
 
 from math import log
 import numpy as np
 
+from scipy.special import expit
+from scipy.special import xlogy
 from scipy.optimize import fmin_bfgs
-from sklearn.preprocessing import LabelEncoder
+from .preprocessing import LabelEncoder
 
-from .base import BaseEstimator, ClassifierMixin, RegressorMixin, clone
+from .base import (BaseEstimator, ClassifierMixin, RegressorMixin, clone,
+                   MetaEstimatorMixin)
 from .preprocessing import label_binarize, LabelBinarizer
 from .utils import check_X_y, check_array, indexable, column_or_1d
 from .utils.validation import check_is_fitted, check_consistent_length
-from .utils.fixes import signature
+from .utils.validation import _check_sample_weight
 from .isotonic import IsotonicRegression
 from .svm import LinearSVC
 from .model_selection import check_cv
-from .metrics.classification import _check_binary_probabilistic_predictions
 
 
-class CalibratedClassifierCV(BaseEstimator, ClassifierMixin):
+class CalibratedClassifierCV(BaseEstimator, ClassifierMixin,
+                             MetaEstimatorMixin):
     """Probability calibration with isotonic regression or sigmoid.
 
+    See glossary entry for :term:`cross-validation estimator`.
+
     With this class, the base_estimator is fit on the train set of the
     cross-validation generator and the test set is used for calibration.
     The probabilities for each of the folds are then averaged
@@ -59,10 +64,10 @@ class CalibratedClassifierCV(BaseEstimator, ClassifierMixin):
         Determines the cross-validation splitting strategy.
         Possible inputs for cv are:
 
-        - None, to use the default 3-fold cross-validation,
+        - None, to use the default 5-fold cross-validation,
         - integer, to specify the number of folds.
-        - An object to be used as a cross-validation generator.
-        - An iterable yielding train/test splits.
+        - :term:`CV splitter`,
+        - An iterable yielding (train, test) splits as arrays of indices.
 
         For integer/None inputs, if ``y`` is binary or multiclass,
         :class:`sklearn.model_selection.StratifiedKFold` is used. If ``y`` is
@@ -75,9 +80,8 @@ class CalibratedClassifierCV(BaseEstimator, ClassifierMixin):
         If "prefit" is passed, it is assumed that base_estimator has been
         fitted already and all data is used for calibration.
 
-        .. versionchanged:: 0.20
-            ``cv`` default value if None will change from 3-fold to 5-fold
-            in v0.22.
+        .. versionchanged:: 0.22
+            ``cv`` default value if None changed from 3-fold to 5-fold.
 
     Attributes
     ----------
@@ -103,7 +107,7 @@ class CalibratedClassifierCV(BaseEstimator, ClassifierMixin):
     .. [4] Predicting Good Probabilities with Supervised Learning,
            A. Niculescu-Mizil & R. Caruana, ICML 2005
     """
-    def __init__(self, base_estimator=None, method='sigmoid', cv='warn'):
+    def __init__(self, base_estimator=None, method='sigmoid', cv=None):
         self.base_estimator = base_estimator
         self.method = method
         self.cv = cv
@@ -119,7 +123,7 @@ class CalibratedClassifierCV(BaseEstimator, ClassifierMixin):
         y : array-like, shape (n_samples,)
             Target values.
 
-        sample_weight : array-like, shape = [n_samples] or None
+        sample_weight : array-like of shape (n_samples,), default=None
             Sample weights. If None, then samples are equally weighted.
 
         Returns
@@ -128,7 +132,7 @@ class CalibratedClassifierCV(BaseEstimator, ClassifierMixin):
             Returns an instance of self.
         """
         X, y = check_X_y(X, y, accept_sparse=['csc', 'csr', 'coo'],
-                         force_all_finite=False)
+                         force_all_finite=False, allow_nd=True)
         X, y = indexable(X, y)
         le = LabelBinarizer().fit(y)
         self.classes_ = le.classes_
@@ -169,11 +173,11 @@ class CalibratedClassifierCV(BaseEstimator, ClassifierMixin):
                 warnings.warn("%s does not support sample_weight. Samples"
                               " weights are only used for the calibration"
                               " itself." % estimator_name)
+                sample_weight = check_array(sample_weight, ensure_2d=False)
                 base_estimator_sample_weight = None
             else:
                 if sample_weight is not None:
-                    sample_weight = check_array(sample_weight, ensure_2d=False)
-                    check_consistent_length(y, sample_weight)
+                    sample_weight = _check_sample_weight(sample_weight, X)
                 base_estimator_sample_weight = sample_weight
             for train, test in cv.split(X, y):
                 this_estimator = clone(base_estimator)
@@ -212,7 +216,7 @@ class CalibratedClassifierCV(BaseEstimator, ClassifierMixin):
         C : array, shape (n_samples, n_classes)
             The predicted probas.
         """
-        check_is_fitted(self, ["classes_", "calibrated_classifiers_"])
+        check_is_fitted(self)
         X = check_array(X, accept_sparse=['csc', 'csr', 'coo'],
                         force_all_finite=False)
         # Compute the arithmetic mean of the predictions of the calibrated
@@ -240,11 +244,11 @@ class CalibratedClassifierCV(BaseEstimator, ClassifierMixin):
         C : array, shape (n_samples,)
             The predicted class.
         """
-        check_is_fitted(self, ["classes_", "calibrated_classifiers_"])
+        check_is_fitted(self)
         return self.classes_[np.argmax(self.predict_proba(X), axis=1)]
 
 
-class _CalibratedClassifier(object):
+class _CalibratedClassifier:
     """Probability calibration with isotonic regression or sigmoid.
 
     It assumes that base_estimator has already been fit, and trains the
@@ -322,7 +326,7 @@ class _CalibratedClassifier(object):
         y : array-like, shape (n_samples,)
             Target values.
 
-        sample_weight : array-like, shape = [n_samples] or None
+        sample_weight : array-like of shape (n_samples,), default=None
             Sample weights. If None, then samples are equally weighted.
 
         Returns
@@ -409,7 +413,7 @@ def _sigmoid_calibration(df, y, sample_weight=None):
     y : ndarray, shape (n_samples,)
         The targets.
 
-    sample_weight : array-like, shape = [n_samples] or None
+    sample_weight : array-like of shape (n_samples,), default=None
         Sample weights. If None, then samples are equally weighted.
 
     Returns
@@ -428,7 +432,6 @@ def _sigmoid_calibration(df, y, sample_weight=None):
     y = column_or_1d(y)
 
     F = df  # F follows Platt's notations
-    tiny = np.finfo(np.float).tiny  # to avoid division by 0 warning
 
     # Bayesian priors (see Platt end of section 2.2)
     prior0 = float(np.sum(y <= 0))
@@ -440,19 +443,17 @@ def _sigmoid_calibration(df, y, sample_weight=None):
 
     def objective(AB):
         # From Platt (beginning of Section 2.2)
-        E = np.exp(AB[0] * F + AB[1])
-        P = 1. / (1. + E)
-        l = -(T * np.log(P + tiny) + T1 * np.log(1. - P + tiny))
+        P = expit(-(AB[0] * F + AB[1]))
+        loss = -(xlogy(T, P) + xlogy(T1, 1. - P))
         if sample_weight is not None:
-            return (sample_weight * l).sum()
+            return (sample_weight * loss).sum()
         else:
-            return l.sum()
+            return loss.sum()
 
     def grad(AB):
         # gradient of the objective function
-        E = np.exp(AB[0] * F + AB[1])
-        P = 1. / (1. + E)
-        TEP_minus_T1P = P * (T * E - T1)
+        P = expit(-(AB[0] * F + AB[1]))
+        TEP_minus_T1P = T - P
         if sample_weight is not None:
             TEP_minus_T1P *= sample_weight
         dA = np.dot(TEP_minus_T1P, F)
@@ -464,7 +465,7 @@ def _sigmoid_calibration(df, y, sample_weight=None):
     return AB_[0], AB_[1]
 
 
-class _SigmoidCalibration(BaseEstimator, RegressorMixin):
+class _SigmoidCalibration(RegressorMixin, BaseEstimator):
     """Sigmoid regression model.
 
     Attributes
@@ -486,7 +487,7 @@ class _SigmoidCalibration(BaseEstimator, RegressorMixin):
         y : array-like, shape (n_samples,)
             Training target.
 
-        sample_weight : array-like, shape = [n_samples] or None
+        sample_weight : array-like of shape (n_samples,), default=None
             Sample weights. If None, then samples are equally weighted.
 
         Returns
@@ -515,15 +516,16 @@ class _SigmoidCalibration(BaseEstimator, RegressorMixin):
             The predicted data.
         """
         T = column_or_1d(T)
-        return 1. / (1. + np.exp(self.a_ * T + self.b_))
+        return expit(-(self.a_ * T + self.b_))
 
 
-def calibration_curve(y_true, y_prob, normalize=False, n_bins=5):
+def calibration_curve(y_true, y_prob, normalize=False, n_bins=5,
+                      strategy='uniform'):
     """Compute true and predicted probabilities for a calibration curve.
 
-     The method assumes the inputs come from a binary classifier.
+    The method assumes the inputs come from a binary classifier.
 
-     Calibration curves may also be referred to as reliability diagrams.
+    Calibration curves may also be referred to as reliability diagrams.
 
     Read more in the :ref:`User Guide <calibration>`.
 
@@ -541,14 +543,24 @@ def calibration_curve(y_true, y_prob, normalize=False, n_bins=5):
         onto 0 and the largest one onto 1.
 
     n_bins : int
-        Number of bins. A bigger number requires more data.
+        Number of bins. A bigger number requires more data. Bins with no data
+        points (i.e. without corresponding values in y_prob) will not be
+        returned, thus there may be fewer than n_bins in the return value.
+
+    strategy : {'uniform', 'quantile'}, (default='uniform')
+        Strategy used to define the widths of the bins.
+
+        uniform
+            All bins have identical widths.
+        quantile
+            All bins have the same number of points.
 
     Returns
     -------
-    prob_true : array, shape (n_bins,)
+    prob_true : array, shape (n_bins,) or smaller
         The true probability in each bin (fraction of positives).
 
-    prob_pred : array, shape (n_bins,)
+    prob_pred : array, shape (n_bins,) or smaller
         The mean predicted probability in each bin.
 
     References
@@ -560,6 +572,7 @@ def calibration_curve(y_true, y_prob, normalize=False, n_bins=5):
     """
     y_true = column_or_1d(y_true)
     y_prob = column_or_1d(y_prob)
+    check_consistent_length(y_true, y_prob)
 
     if normalize:  # Normalize predicted values into interval [0, 1]
         y_prob = (y_prob - y_prob.min()) / (y_prob.max() - y_prob.min())
@@ -567,9 +580,22 @@ def calibration_curve(y_true, y_prob, normalize=False, n_bins=5):
         raise ValueError("y_prob has values outside [0, 1] and normalize is "
                          "set to False.")
 
-    y_true = _check_binary_probabilistic_predictions(y_true, y_prob)
+    labels = np.unique(y_true)
+    if len(labels) > 2:
+        raise ValueError("Only binary classification is supported. "
+                         "Provided labels %s." % labels)
+    y_true = label_binarize(y_true, labels)[:, 0]
+
+    if strategy == 'quantile':  # Determine bin edges by distribution of data
+        quantiles = np.linspace(0, 1, n_bins + 1)
+        bins = np.percentile(y_prob, quantiles * 100)
+        bins[-1] = bins[-1] + 1e-8
+    elif strategy == 'uniform':
+        bins = np.linspace(0., 1. + 1e-8, n_bins + 1)
+    else:
+        raise ValueError("Invalid entry to 'strategy' input. Strategy "
+                         "must be either 'quantile' or 'uniform'.")
 
-    bins = np.linspace(0., 1. + 1e-8, n_bins + 1)
     binids = np.digitize(y_prob, bins) - 1
 
     bin_sums = np.bincount(binids, weights=y_prob, minlength=len(bins))
