@@ -2,24 +2,25 @@
 #         Arnaud Joly <a.joly@ulg.ac.be>
 #         Maheshakya Wijewardena <maheshakya.10@cse.mrt.ac.lk>
 # License: BSD 3 clause
-from __future__ import division
 
 import warnings
 import numpy as np
 import scipy.sparse as sp
 
 from .base import BaseEstimator, ClassifierMixin, RegressorMixin
+from .base import MultiOutputMixin
 from .utils import check_random_state
 from .utils.validation import _num_samples
 from .utils.validation import check_array
 from .utils.validation import check_consistent_length
-from .utils.validation import check_is_fitted
-from .utils.random import random_choice_csc
+from .utils.validation import check_is_fitted, _check_sample_weight
+from .utils.random import _random_choice_csc
 from .utils.stats import _weighted_percentile
 from .utils.multiclass import class_distribution
+from .utils import deprecated
 
 
-class DummyClassifier(BaseEstimator, ClassifierMixin):
+class DummyClassifier(MultiOutputMixin, ClassifierMixin, BaseEstimator):
     """
     DummyClassifier is a classifier that makes predictions using simple rules.
 
@@ -28,6 +29,8 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
 
     Read more in the :ref:`User Guide <dummy_estimators>`.
 
+    .. versionadded:: 0.13
+
     Parameters
     ----------
     strategy : str, default="stratified"
@@ -44,6 +47,11 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
           the user. This is useful for metrics that evaluate a non-majority
           class
 
+          .. versionchanged:: 0.22
+             The default value of `strategy` will change to "prior" in version
+             0.24. Starting from version 0.22, a warning will be raised if
+             `strategy` is not explicitly set.
+
           .. versionadded:: 0.17
              Dummy Classifier now supports prior fitting strategy using
              parameter *prior*.
@@ -54,34 +62,44 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
         If None, the random number generator is the RandomState instance used
         by `np.random`.
 
-    constant : int or str or array of shape = [n_outputs]
+    constant : int or str or array-like of shape (n_outputs,)
         The explicit constant as predicted by the "constant" strategy. This
         parameter is useful only for the "constant" strategy.
 
     Attributes
     ----------
-    classes_ : array or list of array of shape = [n_classes]
+    classes_ : array or list of array of shape (n_classes,)
         Class labels for each output.
 
-    n_classes_ : array or list of array of shape = [n_classes]
+    n_classes_ : array or list of array of shape (n_classes,)
         Number of label for each output.
 
-    class_prior_ : array or list of array of shape = [n_classes]
+    class_prior_ : array or list of array of shape (n_classes,)
         Probability of each class for each output.
 
     n_outputs_ : int,
         Number of outputs.
 
-    outputs_2d_ : bool,
-        True if the output at fit is 2d, else false.
-
     sparse_output_ : bool,
         True if the array returned from predict is to be in sparse CSC format.
         Is automatically set to True if the input y is passed in sparse format.
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> from sklearn.dummy import DummyClassifier
+    >>> X = np.array([-1, 1, 1, 1])
+    >>> y = np.array([0, 1, 1, 1])
+    >>> dummy_clf = DummyClassifier(strategy="most_frequent")
+    >>> dummy_clf.fit(X, y)
+    DummyClassifier(strategy='most_frequent')
+    >>> dummy_clf.predict(X)
+    array([1, 1, 1, 1])
+    >>> dummy_clf.score(X, y)
+    0.75
     """
 
-    def __init__(self, strategy="stratified", random_state=None,
+    def __init__(self, strategy="warn", random_state=None,
                  constant=None):
         self.strategy = strategy
         self.random_state = random_state
@@ -95,10 +113,10 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
         X : {array-like, object with finite length or shape}
             Training data, requires length = n_samples
 
-        y : array-like, shape = [n_samples] or [n_samples, n_outputs]
+        y : array-like of shape (n_samples,) or (n_samples, n_outputs)
             Target values.
 
-        sample_weight : array-like of shape = [n_samples], optional
+        sample_weight : array-like of shape (n_samples,), default=None
             Sample weights.
 
         Returns
@@ -107,11 +125,19 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
         """
         allowed_strategies = ("most_frequent", "stratified", "uniform",
                               "constant", "prior")
-        if self.strategy not in allowed_strategies:
+
+        # TODO: Remove in 0.24
+        if self.strategy == "warn":
+            warnings.warn("The default value of strategy will change from "
+                          "stratified to prior in 0.24.", FutureWarning)
+            self._strategy = "stratified"
+        elif self.strategy not in allowed_strategies:
             raise ValueError("Unknown strategy type: %s, expected one of %s."
                              % (self.strategy, allowed_strategies))
+        else:
+            self._strategy = self.strategy
 
-        if self.strategy == "uniform" and sp.issparse(y):
+        if self._strategy == "uniform" and sp.issparse(y):
             y = y.toarray()
             warnings.warn('A local copy of the target data has been converted '
                           'to a numpy array. Predicting on sparse target data '
@@ -122,9 +148,9 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
         self.sparse_output_ = sp.issparse(y)
 
         if not self.sparse_output_:
+            y = np.asarray(y)
             y = np.atleast_1d(y)
 
-        self.output_2d_ = y.ndim == 2
         if y.ndim == 1:
             y = np.reshape(y, (-1, 1))
 
@@ -132,7 +158,10 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
 
         check_consistent_length(X, y, sample_weight)
 
-        if self.strategy == "constant":
+        if sample_weight is not None:
+            sample_weight = _check_sample_weight(sample_weight, X)
+
+        if self._strategy == "constant":
             if self.constant is None:
                 raise ValueError("Constant target value has to be specified "
                                  "when the constant strategy is used.")
@@ -146,15 +175,18 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
          self.n_classes_,
          self.class_prior_) = class_distribution(y, sample_weight)
 
-        if (self.strategy == "constant" and
-                any(constant[k] not in self.classes_[k]
-                    for k in range(self.n_outputs_))):
-            # Checking in case of constant strategy if the constant
-            # provided by the user is in y.
-            raise ValueError("The constant target value must be "
-                             "present in training data")
+        if self._strategy == "constant":
+            for k in range(self.n_outputs_):
+                if not any(constant[k][0] == c for c in self.classes_[k]):
+                    # Checking in case of constant strategy if the constant
+                    # provided by the user is in y.
+                    err_msg = ("The constant target value must be present in "
+                               "the training data. You provided constant={}. "
+                               "Possible values are: {}."
+                               .format(self.constant, list(self.classes_[k])))
+                    raise ValueError(err_msg)
 
-        if self.n_outputs_ == 1 and not self.output_2d_:
+        if self.n_outputs_ == 1:
             self.n_classes_ = self.n_classes_[0]
             self.classes_ = self.classes_[0]
             self.class_prior_ = self.class_prior_[0]
@@ -171,10 +203,10 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
 
         Returns
         -------
-        y : array, shape = [n_samples] or [n_samples, n_outputs]
+        y : array-like of shape (n_samples,) or (n_samples, n_outputs)
             Predicted target values for X.
         """
-        check_is_fitted(self, 'classes_')
+        check_is_fitted(self)
 
         # numpy random_state expects Python int and not long as size argument
         # under Windows
@@ -192,46 +224,46 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
             class_prior_ = [class_prior_]
             constant = [constant]
         # Compute probability only once
-        if self.strategy == "stratified":
+        if self._strategy == "stratified":
             proba = self.predict_proba(X)
             if self.n_outputs_ == 1:
                 proba = [proba]
 
         if self.sparse_output_:
             class_prob = None
-            if self.strategy in ("most_frequent", "prior"):
+            if self._strategy in ("most_frequent", "prior"):
                 classes_ = [np.array([cp.argmax()]) for cp in class_prior_]
 
-            elif self.strategy == "stratified":
+            elif self._strategy == "stratified":
                 class_prob = class_prior_
 
-            elif self.strategy == "uniform":
+            elif self._strategy == "uniform":
                 raise ValueError("Sparse target prediction is not "
                                  "supported with the uniform strategy")
 
-            elif self.strategy == "constant":
+            elif self._strategy == "constant":
                 classes_ = [np.array([c]) for c in constant]
 
-            y = random_choice_csc(n_samples, classes_, class_prob,
+            y = _random_choice_csc(n_samples, classes_, class_prob,
                                   self.random_state)
         else:
-            if self.strategy in ("most_frequent", "prior"):
+            if self._strategy in ("most_frequent", "prior"):
                 y = np.tile([classes_[k][class_prior_[k].argmax()] for
                              k in range(self.n_outputs_)], [n_samples, 1])
 
-            elif self.strategy == "stratified":
+            elif self._strategy == "stratified":
                 y = np.vstack([classes_[k][proba[k].argmax(axis=1)] for
                                k in range(self.n_outputs_)]).T
 
-            elif self.strategy == "uniform":
+            elif self._strategy == "uniform":
                 ret = [classes_[k][rs.randint(n_classes_[k], size=n_samples)]
                        for k in range(self.n_outputs_)]
                 y = np.vstack(ret).T
 
-            elif self.strategy == "constant":
+            elif self._strategy == "constant":
                 y = np.tile(self.constant, (n_samples, 1))
 
-            if self.n_outputs_ == 1 and not self.output_2d_:
+            if self.n_outputs_ == 1:
                 y = np.ravel(y)
 
         return y
@@ -247,12 +279,12 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
 
         Returns
         -------
-        P : array-like or list of array-lke of shape = [n_samples, n_classes]
+        P : array-like or list of array-lke of shape (n_samples, n_classes)
             Returns the probability of the sample for each class in
             the model, where classes are ordered arithmetically, for each
             output.
         """
-        check_is_fitted(self, 'classes_')
+        check_is_fitted(self)
 
         # numpy random_state expects Python int and not long as size argument
         # under Windows
@@ -263,7 +295,7 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
         classes_ = self.classes_
         class_prior_ = self.class_prior_
         constant = self.constant
-        if self.n_outputs_ == 1 and not self.output_2d_:
+        if self.n_outputs_ == 1:
             # Get same type even for self.n_outputs_ == 1
             n_classes_ = [n_classes_]
             classes_ = [classes_]
@@ -272,28 +304,29 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
 
         P = []
         for k in range(self.n_outputs_):
-            if self.strategy == "most_frequent":
+            if self._strategy == "most_frequent":
                 ind = class_prior_[k].argmax()
                 out = np.zeros((n_samples, n_classes_[k]), dtype=np.float64)
                 out[:, ind] = 1.0
-            elif self.strategy == "prior":
+            elif self._strategy == "prior":
                 out = np.ones((n_samples, 1)) * class_prior_[k]
 
-            elif self.strategy == "stratified":
+            elif self._strategy == "stratified":
                 out = rs.multinomial(1, class_prior_[k], size=n_samples)
+                out = out.astype(np.float64)
 
-            elif self.strategy == "uniform":
+            elif self._strategy == "uniform":
                 out = np.ones((n_samples, n_classes_[k]), dtype=np.float64)
                 out /= n_classes_[k]
 
-            elif self.strategy == "constant":
+            elif self._strategy == "constant":
                 ind = np.where(classes_[k] == constant[k])
                 out = np.zeros((n_samples, n_classes_[k]), dtype=np.float64)
                 out[:, ind] = 1.0
 
             P.append(out)
 
-        if self.n_outputs_ == 1 and not self.output_2d_:
+        if self.n_outputs_ == 1:
             P = P[0]
 
         return P
@@ -309,7 +342,7 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
 
         Returns
         -------
-        P : array-like or list of array-like of shape = [n_samples, n_classes]
+        P : array-like or list of array-like of shape (n_samples, n_classes)
             Returns the log probability of the sample for each class in
             the model, where classes are ordered arithmetically for each
             output.
@@ -320,6 +353,9 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
         else:
             return [np.log(p) for p in proba]
 
+    def _more_tags(self):
+        return {'poor_score': True, 'no_validation': True}
+
     def score(self, X, y, sample_weight=None):
         """Returns the mean accuracy on the given test data and labels.
 
@@ -335,10 +371,10 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
             as passing real test samples, since DummyClassifier
             operates independently of the sampled observations.
 
-        y : array-like, shape = (n_samples) or (n_samples, n_outputs)
+        y : array-like of shape (n_samples,) or (n_samples, n_outputs)
             True labels for X.
 
-        sample_weight : array-like, shape = [n_samples], optional
+        sample_weight : array-like of shape (n_samples,), default=None
             Sample weights.
 
         Returns
@@ -349,10 +385,19 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
         """
         if X is None:
             X = np.zeros(shape=(len(y), 1))
-        return super(DummyClassifier, self).score(X, y, sample_weight)
+        return super().score(X, y, sample_weight)
+
+    @deprecated(
+        "The outputs_2d_ attribute is deprecated in version 0.22 "
+        "and will be removed in version 0.24. It is equivalent to "
+        "n_outputs_ > 1."
+    )
+    @property
+    def outputs_2d_(self):
+        return self.n_outputs_ != 1
 
 
-class DummyRegressor(BaseEstimator, RegressorMixin):
+class DummyRegressor(MultiOutputMixin, RegressorMixin, BaseEstimator):
     """
     DummyRegressor is a regressor that makes predictions using
     simple rules.
@@ -362,6 +407,8 @@ class DummyRegressor(BaseEstimator, RegressorMixin):
 
     Read more in the :ref:`User Guide <dummy_estimators>`.
 
+    .. versionadded:: 0.13
+
     Parameters
     ----------
     strategy : str
@@ -374,7 +421,7 @@ class DummyRegressor(BaseEstimator, RegressorMixin):
         * "constant": always predicts a constant value that is provided by
           the user.
 
-    constant : int or float or array of shape = [n_outputs]
+    constant : int or float or array-like of shape (n_outputs,)
         The explicit constant as predicted by the "constant" strategy. This
         parameter is useful only for the "constant" strategy.
 
@@ -385,15 +432,26 @@ class DummyRegressor(BaseEstimator, RegressorMixin):
 
     Attributes
     ----------
-    constant_ : float or array of shape [n_outputs]
+    constant_ : array, shape (1, n_outputs)
         Mean or median or quantile of the training targets or constant value
         given by the user.
 
     n_outputs_ : int,
         Number of outputs.
 
-    outputs_2d_ : bool,
-        True if the output at fit is 2d, else false.
+    Examples
+    --------
+    >>> import numpy as np
+    >>> from sklearn.dummy import DummyRegressor
+    >>> X = np.array([1.0, 2.0, 3.0, 4.0])
+    >>> y = np.array([2.0, 3.0, 5.0, 10.0])
+    >>> dummy_regr = DummyRegressor(strategy="mean")
+    >>> dummy_regr.fit(X, y)
+    DummyRegressor()
+    >>> dummy_regr.predict(X)
+    array([5., 5., 5., 5.])
+    >>> dummy_regr.score(X, y)
+    0.0
     """
 
     def __init__(self, strategy="mean", constant=None, quantile=None):
@@ -409,10 +467,10 @@ class DummyRegressor(BaseEstimator, RegressorMixin):
         X : {array-like, object with finite length or shape}
             Training data, requires length = n_samples
 
-        y : array-like, shape = [n_samples] or [n_samples, n_outputs]
+        y : array-like of shape (n_samples,) or (n_samples, n_outputs)
             Target values.
 
-        sample_weight : array-like of shape = [n_samples], optional
+        sample_weight : array-like of shape (n_samples,), default=None
             Sample weights.
 
         Returns
@@ -428,13 +486,15 @@ class DummyRegressor(BaseEstimator, RegressorMixin):
         if len(y) == 0:
             raise ValueError("y must not be empty.")
 
-        self.output_2d_ = y.ndim == 2
         if y.ndim == 1:
             y = np.reshape(y, (-1, 1))
         self.n_outputs_ = y.shape[1]
 
         check_consistent_length(X, y, sample_weight)
 
+        if sample_weight is not None:
+            sample_weight = _check_sample_weight(sample_weight, X)
+
         if self.strategy == "mean":
             self.constant_ = np.average(y, axis=0, weights=sample_weight)
 
@@ -468,7 +528,7 @@ class DummyRegressor(BaseEstimator, RegressorMixin):
                                         accept_sparse=['csr', 'csc', 'coo'],
                                         ensure_2d=False, ensure_min_samples=0)
 
-            if self.output_2d_ and self.constant.shape[0] != y.shape[1]:
+            if self.n_outputs_ != 1 and self.constant.shape[0] != y.shape[1]:
                 raise ValueError(
                     "Constant target value should have "
                     "shape (%d, 1)." % y.shape[1])
@@ -493,25 +553,28 @@ class DummyRegressor(BaseEstimator, RegressorMixin):
 
         Returns
         -------
-        y : array, shape = [n_samples]  or [n_samples, n_outputs]
+        y : array-like of shape (n_samples,) or (n_samples, n_outputs)
             Predicted target values for X.
 
-        y_std : array, shape = [n_samples]  or [n_samples, n_outputs]
+        y_std : array-like of shape (n_samples,) or (n_samples, n_outputs)
             Standard deviation of predictive distribution of query points.
         """
-        check_is_fitted(self, "constant_")
+        check_is_fitted(self)
         n_samples = _num_samples(X)
 
         y = np.full((n_samples, self.n_outputs_), self.constant_,
                     dtype=np.array(self.constant_).dtype)
         y_std = np.zeros((n_samples, self.n_outputs_))
 
-        if self.n_outputs_ == 1 and not self.output_2d_:
+        if self.n_outputs_ == 1:
             y = np.ravel(y)
             y_std = np.ravel(y_std)
 
         return (y, y_std) if return_std else y
 
+    def _more_tags(self):
+        return {'poor_score': True, 'no_validation': True}
+
     def score(self, X, y, sample_weight=None):
         """Returns the coefficient of determination R^2 of the prediction.
 
@@ -535,10 +598,10 @@ class DummyRegressor(BaseEstimator, RegressorMixin):
             as passing real test samples, since DummyRegressor
             operates independently of the sampled observations.
 
-        y : array-like, shape = (n_samples) or (n_samples, n_outputs)
+        y : array-like of shape (n_samples,) or (n_samples, n_outputs)
             True values for X.
 
-        sample_weight : array-like, shape = [n_samples], optional
+        sample_weight : array-like of shape (n_samples,), default=None
             Sample weights.
 
         Returns
@@ -548,4 +611,13 @@ class DummyRegressor(BaseEstimator, RegressorMixin):
         """
         if X is None:
             X = np.zeros(shape=(len(y), 1))
-        return super(DummyRegressor, self).score(X, y, sample_weight)
+        return super().score(X, y, sample_weight)
+
+    @deprecated(
+        "The outputs_2d_ attribute is deprecated in version 0.22 "
+        "and will be removed in version 0.24. It is equivalent to "
+        "n_outputs_ > 1."
+    )
+    @property
+    def outputs_2d_(self):
+        return self.n_outputs_ != 1
