@@ -5,25 +5,22 @@ from numpy import linalg
 
 from scipy.sparse import dok_matrix, csr_matrix, issparse
 from scipy.spatial.distance import cosine, cityblock, minkowski, wminkowski
+from scipy.spatial.distance import cdist, pdist, squareform
 
 import pytest
 
-from sklearn.utils.testing import assert_greater
-from sklearn.utils.testing import assert_array_almost_equal
-from sklearn.utils.testing import assert_allclose
-from sklearn.utils.testing import assert_almost_equal
-from sklearn.utils.testing import assert_equal
-from sklearn.utils.testing import assert_array_equal
-from sklearn.utils.testing import assert_raises
-from sklearn.utils.testing import assert_raises_regexp
-from sklearn.utils.testing import assert_warns
-from sklearn.utils.testing import ignore_warnings
-from sklearn.utils.testing import assert_warns_message
+from sklearn import config_context
 
-from sklearn.externals.six import iteritems
+from sklearn.utils._testing import assert_array_almost_equal
+from sklearn.utils._testing import assert_allclose
+from sklearn.utils._testing import assert_almost_equal
+from sklearn.utils._testing import assert_array_equal
+from sklearn.utils._testing import ignore_warnings
 
 from sklearn.metrics.pairwise import euclidean_distances
+from sklearn.metrics.pairwise import nan_euclidean_distances
 from sklearn.metrics.pairwise import manhattan_distances
+from sklearn.metrics.pairwise import haversine_distances
 from sklearn.metrics.pairwise import linear_kernel
 from sklearn.metrics.pairwise import chi2_kernel, additive_chi2_kernel
 from sklearn.metrics.pairwise import polynomial_kernel
@@ -46,55 +43,81 @@ from sklearn.metrics.pairwise import check_paired_arrays
 from sklearn.metrics.pairwise import paired_distances
 from sklearn.metrics.pairwise import paired_euclidean_distances
 from sklearn.metrics.pairwise import paired_manhattan_distances
+from sklearn.metrics.pairwise import _euclidean_distances_upcast
 from sklearn.preprocessing import normalize
 from sklearn.exceptions import DataConversionWarning
 
-import pytest
-
 
 def test_pairwise_distances():
     # Test the pairwise_distance helper function.
     rng = np.random.RandomState(0)
+
     # Euclidean distance should be equivalent to calling the function.
     X = rng.random_sample((5, 4))
     S = pairwise_distances(X, metric="euclidean")
     S2 = euclidean_distances(X)
     assert_array_almost_equal(S, S2)
+
     # Euclidean distance, with Y != X.
     Y = rng.random_sample((2, 4))
     S = pairwise_distances(X, Y, metric="euclidean")
     S2 = euclidean_distances(X, Y)
     assert_array_almost_equal(S, S2)
+    # Check to ensure NaNs work with pairwise_distances.
+    X_masked = rng.random_sample((5, 4))
+    Y_masked = rng.random_sample((2, 4))
+    X_masked[0, 0] = np.nan
+    Y_masked[0, 0] = np.nan
+    S_masked = pairwise_distances(X_masked, Y_masked, metric="nan_euclidean")
+    S2_masked = nan_euclidean_distances(X_masked, Y_masked)
+    assert_array_almost_equal(S_masked, S2_masked)
     # Test with tuples as X and Y
     X_tuples = tuple([tuple([v for v in row]) for row in X])
     Y_tuples = tuple([tuple([v for v in row]) for row in Y])
     S2 = pairwise_distances(X_tuples, Y_tuples, metric="euclidean")
     assert_array_almost_equal(S, S2)
+
+    # Test haversine distance
+    # The data should be valid latitude and longitude
+    X = rng.random_sample((5, 2))
+    X[:, 0] = (X[:, 0] - 0.5) * 2 * np.pi/2
+    X[:, 1] = (X[:, 1] - 0.5) * 2 * np.pi
+    S = pairwise_distances(X, metric="haversine")
+    S2 = haversine_distances(X)
+    assert_array_almost_equal(S, S2)
+
+    # Test haversine distance, with Y != X
+    Y = rng.random_sample((2, 2))
+    Y[:, 0] = (Y[:, 0] - 0.5)*2*np.pi/2
+    Y[:, 1] = (Y[:, 1] - 0.5)*2*np.pi
+    S = pairwise_distances(X, Y, metric="haversine")
+    S2 = haversine_distances(X, Y)
+    assert_array_almost_equal(S, S2)
+
     # "cityblock" uses scikit-learn metric, cityblock (function) is
     # scipy.spatial.
     S = pairwise_distances(X, metric="cityblock")
     S2 = pairwise_distances(X, metric=cityblock)
-    assert_equal(S.shape[0], S.shape[1])
-    assert_equal(S.shape[0], X.shape[0])
+    assert S.shape[0] == S.shape[1]
+    assert S.shape[0] == X.shape[0]
     assert_array_almost_equal(S, S2)
+
     # The manhattan metric should be equivalent to cityblock.
     S = pairwise_distances(X, Y, metric="manhattan")
     S2 = pairwise_distances(X, Y, metric=cityblock)
-    assert_equal(S.shape[0], X.shape[0])
-    assert_equal(S.shape[1], Y.shape[0])
+    assert S.shape[0] == X.shape[0]
+    assert S.shape[1] == Y.shape[0]
     assert_array_almost_equal(S, S2)
-    # Using size_threshold argument should raise
-    # a deprecation warning
-    assert_warns(DeprecationWarning,
-                 manhattan_distances, X, Y, size_threshold=10)
+
     # Test cosine as a string metric versus cosine callable
     # The string "cosine" uses sklearn.metric,
     # while the function cosine is scipy.spatial
     S = pairwise_distances(X, Y, metric="cosine")
     S2 = pairwise_distances(X, Y, metric=cosine)
-    assert_equal(S.shape[0], X.shape[0])
-    assert_equal(S.shape[1], Y.shape[0])
+    assert S.shape[0] == X.shape[0]
+    assert S.shape[1] == Y.shape[0]
     assert_array_almost_equal(S, S2)
+
     # Test with sparse X and Y,
     # currently only supported for Euclidean, L1 and cosine.
     X_sparse = csr_matrix(X)
@@ -110,23 +133,28 @@ def test_pairwise_distances():
     assert_array_almost_equal(S, S2)
     S2 = manhattan_distances(X, Y)
     assert_array_almost_equal(S, S2)
+
     # Test with scipy.spatial.distance metric, with a kwd
     kwds = {"p": 2.0}
     S = pairwise_distances(X, Y, metric="minkowski", **kwds)
     S2 = pairwise_distances(X, Y, metric=minkowski, **kwds)
     assert_array_almost_equal(S, S2)
+
     # same with Y = None
     kwds = {"p": 2.0}
     S = pairwise_distances(X, metric="minkowski", **kwds)
     S2 = pairwise_distances(X, metric=minkowski, **kwds)
     assert_array_almost_equal(S, S2)
+
     # Test that scipy distance metrics throw an error if sparse matrix given
-    assert_raises(TypeError, pairwise_distances, X_sparse, metric="minkowski")
-    assert_raises(TypeError, pairwise_distances, X, Y_sparse,
-                  metric="minkowski")
+    with pytest.raises(TypeError):
+        pairwise_distances(X_sparse, metric="minkowski")
+    with pytest.raises(TypeError):
+        pairwise_distances(X, Y_sparse, metric="minkowski")
 
     # Test that a value error is raised if the metric is unknown
-    assert_raises(ValueError, pairwise_distances, X, Y, metric="blah")
+    with pytest.raises(ValueError):
+        pairwise_distances(X, Y, metric="blah")
 
 
 @pytest.mark.parametrize('metric', PAIRWISE_BOOLEAN_FUNCTIONS)
@@ -144,20 +172,42 @@ def test_pairwise_boolean_distance(metric):
             res[np.isnan(res)] = 0
             assert np.sum(res != 0) == 0
 
+    # non-boolean arrays are converted to boolean for boolean
+    # distance metrics with a data conversion warning
+    msg = "Data was converted to boolean for metric %s" % metric
+    with pytest.warns(DataConversionWarning, match=msg):
+        pairwise_distances(X, metric=metric)
+
+    # Check that the warning is raised if X is boolean by Y is not boolean:
+    with pytest.warns(DataConversionWarning, match=msg):
+        pairwise_distances(X.astype(bool), Y=Y, metric=metric)
+
+    # Check that no warning is raised if X is already boolean and Y is None:
+    with pytest.warns(None) as records:
+        pairwise_distances(X.astype(bool), metric=metric)
+    assert len(records) == 0
+
+
+def test_no_data_conversion_warning():
+    # No warnings issued if metric is not a boolean distance function
+    rng = np.random.RandomState(0)
+    X = rng.randn(5, 4)
+    with pytest.warns(None) as records:
+        pairwise_distances(X, metric="minkowski")
+    assert len(records) == 0
+
 
 @pytest.mark.parametrize('func', [pairwise_distances, pairwise_kernels])
 def test_pairwise_precomputed(func):
     # Test correct shape
-    assert_raises_regexp(ValueError, '.* shape .*',
-                         func, np.zeros((5, 3)), metric='precomputed')
+    with pytest.raises(ValueError, match='.* shape .*'):
+        func(np.zeros((5, 3)), metric='precomputed')
     # with two args
-    assert_raises_regexp(ValueError, '.* shape .*',
-                         func, np.zeros((5, 3)), np.zeros((4, 4)),
-                         metric='precomputed')
+    with pytest.raises(ValueError, match='.* shape .*'):
+        func(np.zeros((5, 3)), np.zeros((4, 4)), metric='precomputed')
     # even if shape[1] agrees (although thus second arg is spurious)
-    assert_raises_regexp(ValueError, '.* shape .*',
-                         func, np.zeros((5, 3)), np.zeros((4, 3)),
-                         metric='precomputed')
+    with pytest.raises(ValueError, match='.* shape .*'):
+        func(np.zeros((5, 3)), np.zeros((4, 3)), metric='precomputed')
 
     # Test not copied (if appropriate dtype)
     S = np.zeros((5, 5))
@@ -170,39 +220,20 @@ def test_pairwise_precomputed(func):
 
     # Test always returns float dtype
     S = func(np.array([[1]], dtype='int'), metric='precomputed')
-    assert_equal('f', S.dtype.kind)
+    assert 'f' == S.dtype.kind
 
     # Test converts list to array-like
     S = func([[1.]], metric='precomputed')
     assert isinstance(S, np.ndarray)
 
 
-def check_pairwise_parallel(func, metric, kwds):
-    rng = np.random.RandomState(0)
-    for make_data in (np.array, csr_matrix):
-        X = make_data(rng.random_sample((5, 4)))
-        Y = make_data(rng.random_sample((3, 4)))
-
-        try:
-            S = func(X, metric=metric, n_jobs=1, **kwds)
-        except (TypeError, ValueError) as exc:
-            # Not all metrics support sparse input
-            # ValueError may be triggered by bad callable
-            if make_data is csr_matrix:
-                assert_raises(type(exc), func, X, metric=metric,
-                              n_jobs=2, **kwds)
-                continue
-            else:
-                raise
-        S2 = func(X, metric=metric, n_jobs=2, **kwds)
-        assert_array_almost_equal(S, S2)
-
-        S = func(X, Y, metric=metric, n_jobs=1, **kwds)
-        S2 = func(X, Y, metric=metric, n_jobs=2, **kwds)
-        assert_array_almost_equal(S, S2)
+def test_pairwise_precomputed_non_negative():
+    # Test non-negative values
+    with pytest.raises(ValueError, match='.* non-negative values.*'):
+        pairwise_distances(np.full((5, 5), -1), metric='precomputed')
 
 
-_wminkowski_kwds = {'w': np.arange(1, 5).astype('double'), 'p': 1}
+_wminkowski_kwds = {'w': np.arange(1, 5).astype('double', copy=False), 'p': 1}
 
 
 def callable_rbf_kernel(x, y, **kwds):
@@ -218,15 +249,37 @@ def callable_rbf_kernel(x, y, **kwds):
          (pairwise_distances, 'wminkowski', _wminkowski_kwds),
          (pairwise_kernels, 'polynomial', {'degree': 1}),
          (pairwise_kernels, callable_rbf_kernel, {'gamma': .1})])
-def test_pairwise_parallel(func, metric, kwds):
-    check_pairwise_parallel(func, metric, kwds)
+@pytest.mark.parametrize('array_constr', [np.array, csr_matrix])
+@pytest.mark.parametrize('dtype', [np.float64, int])
+def test_pairwise_parallel(func, metric, kwds, array_constr, dtype):
+    rng = np.random.RandomState(0)
+    X = array_constr(5 * rng.random_sample((5, 4)), dtype=dtype)
+    Y = array_constr(5 * rng.random_sample((3, 4)), dtype=dtype)
+
+    try:
+        S = func(X, metric=metric, n_jobs=1, **kwds)
+    except (TypeError, ValueError) as exc:
+        # Not all metrics support sparse input
+        # ValueError may be triggered by bad callable
+        if array_constr is csr_matrix:
+            with pytest.raises(type(exc)):
+                func(X, metric=metric, n_jobs=2, **kwds)
+            return
+        else:
+            raise
+    S2 = func(X, metric=metric, n_jobs=2, **kwds)
+    assert_allclose(S, S2)
+
+    S = func(X, Y, metric=metric, n_jobs=1, **kwds)
+    S2 = func(X, Y, metric=metric, n_jobs=2, **kwds)
+    assert_allclose(S, S2)
 
 
 def test_pairwise_callable_nonstrict_metric():
     # paired_distances should allow callable metric where metric(x, x) != 0
     # Knowing that the callable is a strict metric would allow the diagonal to
     # be left uncalculated and set to 0.
-    assert_equal(pairwise_distances([[1.]], metric=lambda x, y: 5)[0, 0], 5)
+    assert pairwise_distances([[1.]], metric=lambda x, y: 5)[0, 0] == 5
 
 
 # Test with all metrics that should be in PAIRWISE_KERNEL_FUNCTIONS.
@@ -260,8 +313,9 @@ def test_pairwise_kernels(metric):
     Y_sparse = csr_matrix(Y)
     if metric in ["chi2", "additive_chi2"]:
         # these don't support sparse matrices yet
-        assert_raises(ValueError, pairwise_kernels,
-                      X_sparse, Y=Y_sparse, metric=metric)
+        with pytest.raises(ValueError):
+            pairwise_kernels(X_sparse, Y=Y_sparse,
+                             metric=metric)
         return
     K1 = pairwise_kernels(X_sparse, Y=Y_sparse, metric=metric)
     assert_array_almost_equal(K1, K2)
@@ -295,10 +349,11 @@ def test_pairwise_kernels_filter_param():
     K2 = pairwise_kernels(X, Y, metric="rbf", filter_params=True, **params)
     assert_array_almost_equal(K, K2)
 
-    assert_raises(TypeError, pairwise_kernels, X, Y, "rbf", **params)
+    with pytest.raises(TypeError):
+        pairwise_kernels(X, Y, "rbf", **params)
 
 
-@pytest.mark.parametrize('metric, func', iteritems(PAIRED_DISTANCES))
+@pytest.mark.parametrize('metric, func', PAIRED_DISTANCES.items())
 def test_paired_distances(metric, func):
     # Test the pairwise_distance helper function.
     rng = np.random.RandomState(0)
@@ -336,7 +391,8 @@ def test_paired_distances_callable():
     # Test that a value error is raised when the lengths of X and Y should not
     # differ
     Y = rng.random_sample((3, 4))
-    assert_raises(ValueError, paired_distances, X, Y)
+    with pytest.raises(ValueError):
+        paired_distances(X, Y)
 
 
 def test_pairwise_distances_argmin_min():
@@ -362,8 +418,8 @@ def test_pairwise_distances_argmin_min():
     assert_array_almost_equal(idxsp, expected_idx)
     assert_array_almost_equal(valssp, expected_vals)
     # We don't want np.matrix here
-    assert_equal(type(idxsp), np.ndarray)
-    assert_equal(type(valssp), np.ndarray)
+    assert type(idxsp) == np.ndarray
+    assert type(valssp) == np.ndarray
 
     # euclidean metric squared
     idx, vals = pairwise_distances_argmin_min(X, Y, metric="euclidean",
@@ -408,11 +464,6 @@ def test_pairwise_distances_argmin_min():
     np.testing.assert_almost_equal(dist_orig_ind, dist_chunked_ind, decimal=7)
     np.testing.assert_almost_equal(dist_orig_val, dist_chunked_val, decimal=7)
 
-    # Test batch_size deprecation warning
-    assert_warns_message(DeprecationWarning, "version 0.22",
-                         pairwise_distances_argmin_min, X, Y, batch_size=500,
-                         metric='euclidean')
-
 
 def _reduce_func(dist, start):
     return dist[:, :100]
@@ -432,6 +483,19 @@ def test_pairwise_distances_chunked_reduce():
     assert_allclose(np.vstack(S_chunks), S, atol=1e-7)
 
 
+def test_pairwise_distances_chunked_reduce_none():
+    # check that the reduce func is allowed to return None
+    rng = np.random.RandomState(0)
+    X = rng.random_sample((10, 4))
+    S_chunks = pairwise_distances_chunked(X, None,
+                                          reduce_func=lambda dist, start: None,
+                                          working_memory=2 ** -16)
+    assert isinstance(S_chunks, GeneratorType)
+    S_chunks = list(S_chunks)
+    assert len(S_chunks) > 1
+    assert all(chunk is None for chunk in S_chunks)
+
+
 @pytest.mark.parametrize('good_reduce', [
     lambda D, start: list(D),
     lambda D, start: np.array(D),
@@ -465,7 +529,8 @@ def test_pairwise_distances_chunked_reduce_invalid(bad_reduce, err_type,
     X = np.arange(10).reshape(-1, 1)
     S_chunks = pairwise_distances_chunked(X, None, reduce_func=bad_reduce,
                                           working_memory=64)
-    assert_raises_regexp(err_type, message, next, S_chunks)
+    with pytest.raises(err_type, match=message):
+        next(S_chunks)
 
 
 def check_pairwise_distances_chunked(X, Y, working_memory, metric='euclidean'):
@@ -497,12 +562,22 @@ def test_pairwise_distances_chunked_diagonal(metric):
     assert_array_almost_equal(np.diag(np.vstack(chunks)), 0, decimal=10)
 
 
+@pytest.mark.parametrize(
+        'metric',
+        ('euclidean', 'l2', 'sqeuclidean'))
+def test_parallel_pairwise_distances_diagonal(metric):
+    rng = np.random.RandomState(0)
+    X = rng.normal(size=(1000, 10), scale=1e10)
+    distances = pairwise_distances(X, metric=metric, n_jobs=2)
+    assert_allclose(np.diag(distances), 0, atol=1e-10)
+
+
 @ignore_warnings
 def test_pairwise_distances_chunked():
     # Test the pairwise_distance helper function.
     rng = np.random.RandomState(0)
     # Euclidean distance should be equivalent to calling the function.
-    X = rng.random_sample((400, 4))
+    X = rng.random_sample((200, 4))
     check_pairwise_distances_chunked(X, None, working_memory=1,
                                      metric='euclidean')
     # Test small amounts of memory
@@ -513,7 +588,7 @@ def test_pairwise_distances_chunked():
     check_pairwise_distances_chunked(X.tolist(), None, working_memory=1,
                                      metric='euclidean')
     # Euclidean distance, with Y != X.
-    Y = rng.random_sample((200, 4))
+    Y = rng.random_sample((100, 4))
     check_pairwise_distances_chunked(X, Y, working_memory=1,
                                      metric='euclidean')
     check_pairwise_distances_chunked(X.tolist(), Y.tolist(), working_memory=1,
@@ -526,8 +601,8 @@ def test_pairwise_distances_chunked():
     check_pairwise_distances_chunked(X, Y, working_memory=1,
                                      metric='cityblock')
     # Test that a value error is raised if the metric is unknown
-    assert_raises(ValueError, next,
-                  pairwise_distances_chunked(X, Y, metric="blah"))
+    with pytest.raises(ValueError):
+        next(pairwise_distances_chunked(X, Y, metric="blah"))
 
     # Test precomputed returns all at once
     D = pairwise_distances(X)
@@ -536,44 +611,294 @@ def test_pairwise_distances_chunked():
                                      metric='precomputed')
     assert isinstance(gen, GeneratorType)
     assert next(gen) is D
-    assert_raises(StopIteration, next, gen)
+    with pytest.raises(StopIteration):
+        next(gen)
 
 
-def test_euclidean_distances():
-    # Check the pairwise Euclidean distances computation
-    X = [[0]]
-    Y = [[1], [2]]
+@pytest.mark.parametrize("x_array_constr", [np.array, csr_matrix],
+                         ids=["dense", "sparse"])
+@pytest.mark.parametrize("y_array_constr", [np.array, csr_matrix],
+                         ids=["dense", "sparse"])
+def test_euclidean_distances_known_result(x_array_constr, y_array_constr):
+    # Check the pairwise Euclidean distances computation on known result
+    X = x_array_constr([[0]])
+    Y = y_array_constr([[1], [2]])
     D = euclidean_distances(X, Y)
-    assert_array_almost_equal(D, [[1., 2.]])
+    assert_allclose(D, [[1., 2.]])
 
-    X = csr_matrix(X)
-    Y = csr_matrix(Y)
-    D = euclidean_distances(X, Y)
-    assert_array_almost_equal(D, [[1., 2.]])
-
-    rng = np.random.RandomState(0)
-    X = rng.random_sample((10, 4))
-    Y = rng.random_sample((20, 4))
-    X_norm_sq = (X ** 2).sum(axis=1).reshape(1, -1)
-    Y_norm_sq = (Y ** 2).sum(axis=1).reshape(1, -1)
 
+@pytest.mark.parametrize("dtype", [np.float32, np.float64])
+@pytest.mark.parametrize("y_array_constr", [np.array, csr_matrix],
+                         ids=["dense", "sparse"])
+def test_euclidean_distances_with_norms(dtype, y_array_constr):
     # check that we still get the right answers with {X,Y}_norm_squared
+    # and that we get a wrong answer with wrong {X,Y}_norm_squared
+    rng = np.random.RandomState(0)
+    X = rng.random_sample((10, 10)).astype(dtype, copy=False)
+    Y = rng.random_sample((20, 10)).astype(dtype, copy=False)
+
+    # norms will only be used if their dtype is float64
+    X_norm_sq = (X.astype(np.float64) ** 2).sum(axis=1).reshape(1, -1)
+    Y_norm_sq = (Y.astype(np.float64) ** 2).sum(axis=1).reshape(1, -1)
+
+    Y = y_array_constr(Y)
+
     D1 = euclidean_distances(X, Y)
     D2 = euclidean_distances(X, Y, X_norm_squared=X_norm_sq)
     D3 = euclidean_distances(X, Y, Y_norm_squared=Y_norm_sq)
     D4 = euclidean_distances(X, Y, X_norm_squared=X_norm_sq,
                              Y_norm_squared=Y_norm_sq)
-    assert_array_almost_equal(D2, D1)
-    assert_array_almost_equal(D3, D1)
-    assert_array_almost_equal(D4, D1)
+    assert_allclose(D2, D1)
+    assert_allclose(D3, D1)
+    assert_allclose(D4, D1)
 
     # check we get the wrong answer with wrong {X,Y}_norm_squared
-    X_norm_sq *= 0.5
-    Y_norm_sq *= 0.5
     wrong_D = euclidean_distances(X, Y,
                                   X_norm_squared=np.zeros_like(X_norm_sq),
                                   Y_norm_squared=np.zeros_like(Y_norm_sq))
-    assert_greater(np.max(np.abs(wrong_D - D1)), .01)
+    with pytest.raises(AssertionError):
+        assert_allclose(wrong_D, D1)
+
+
+@pytest.mark.parametrize("dtype", [np.float32, np.float64])
+@pytest.mark.parametrize("x_array_constr", [np.array, csr_matrix],
+                         ids=["dense", "sparse"])
+@pytest.mark.parametrize("y_array_constr", [np.array, csr_matrix],
+                         ids=["dense", "sparse"])
+def test_euclidean_distances(dtype, x_array_constr, y_array_constr):
+    # check that euclidean distances gives same result as scipy cdist
+    # when X and Y != X are provided
+    rng = np.random.RandomState(0)
+    X = rng.random_sample((100, 10)).astype(dtype, copy=False)
+    X[X < 0.8] = 0
+    Y = rng.random_sample((10, 10)).astype(dtype, copy=False)
+    Y[Y < 0.8] = 0
+
+    expected = cdist(X, Y)
+
+    X = x_array_constr(X)
+    Y = y_array_constr(Y)
+    distances = euclidean_distances(X, Y)
+
+    # the default rtol=1e-7 is too close to the float32 precision
+    # and fails due too rounding errors.
+    assert_allclose(distances, expected, rtol=1e-6)
+    assert distances.dtype == dtype
+
+
+@pytest.mark.parametrize("dtype", [np.float32, np.float64])
+@pytest.mark.parametrize("x_array_constr", [np.array, csr_matrix],
+                         ids=["dense", "sparse"])
+def test_euclidean_distances_sym(dtype, x_array_constr):
+    # check that euclidean distances gives same result as scipy pdist
+    # when only X is provided
+    rng = np.random.RandomState(0)
+    X = rng.random_sample((100, 10)).astype(dtype, copy=False)
+    X[X < 0.8] = 0
+
+    expected = squareform(pdist(X))
+
+    X = x_array_constr(X)
+    distances = euclidean_distances(X)
+
+    # the default rtol=1e-7 is too close to the float32 precision
+    # and fails due too rounding errors.
+    assert_allclose(distances, expected, rtol=1e-6)
+    assert distances.dtype == dtype
+
+
+@pytest.mark.parametrize("batch_size", [None, 5, 7, 101])
+@pytest.mark.parametrize("x_array_constr", [np.array, csr_matrix],
+                         ids=["dense", "sparse"])
+@pytest.mark.parametrize("y_array_constr", [np.array, csr_matrix],
+                         ids=["dense", "sparse"])
+def test_euclidean_distances_upcast(batch_size, x_array_constr,
+                                    y_array_constr):
+    # check batches handling when Y != X (#13910)
+    rng = np.random.RandomState(0)
+    X = rng.random_sample((100, 10)).astype(np.float32)
+    X[X < 0.8] = 0
+    Y = rng.random_sample((10, 10)).astype(np.float32)
+    Y[Y < 0.8] = 0
+
+    expected = cdist(X, Y)
+
+    X = x_array_constr(X)
+    Y = y_array_constr(Y)
+    distances = _euclidean_distances_upcast(X, Y=Y, batch_size=batch_size)
+    distances = np.sqrt(np.maximum(distances, 0))
+
+    # the default rtol=1e-7 is too close to the float32 precision
+    # and fails due too rounding errors.
+    assert_allclose(distances, expected, rtol=1e-6)
+
+
+@pytest.mark.parametrize("batch_size", [None, 5, 7, 101])
+@pytest.mark.parametrize("x_array_constr", [np.array, csr_matrix],
+                         ids=["dense", "sparse"])
+def test_euclidean_distances_upcast_sym(batch_size, x_array_constr):
+    # check batches handling when X is Y (#13910)
+    rng = np.random.RandomState(0)
+    X = rng.random_sample((100, 10)).astype(np.float32)
+    X[X < 0.8] = 0
+
+    expected = squareform(pdist(X))
+
+    X = x_array_constr(X)
+    distances = _euclidean_distances_upcast(X, Y=X, batch_size=batch_size)
+    distances = np.sqrt(np.maximum(distances, 0))
+
+    # the default rtol=1e-7 is too close to the float32 precision
+    # and fails due too rounding errors.
+    assert_allclose(distances, expected, rtol=1e-6)
+
+
+@pytest.mark.parametrize(
+    "dtype, eps, rtol",
+    [(np.float32, 1e-4, 1e-5),
+     pytest.param(
+         np.float64, 1e-8, 0.99,
+         marks=pytest.mark.xfail(reason='failing due to lack of precision'))])
+@pytest.mark.parametrize("dim", [1, 1000000])
+def test_euclidean_distances_extreme_values(dtype, eps, rtol, dim):
+    # check that euclidean distances is correct with float32 input thanks to
+    # upcasting. On float64 there are still precision issues.
+    X = np.array([[1.] * dim], dtype=dtype)
+    Y = np.array([[1. + eps] * dim], dtype=dtype)
+
+    distances = euclidean_distances(X, Y)
+    expected = cdist(X, Y)
+
+    assert_allclose(distances, expected, rtol=1e-5)
+
+
+@pytest.mark.parametrize("squared", [True, False])
+def test_nan_euclidean_distances_equal_to_euclidean_distance(squared):
+    # with no nan values
+    rng = np.random.RandomState(1337)
+    X = rng.randn(3, 4)
+    Y = rng.randn(4, 4)
+
+    normal_distance = euclidean_distances(X, Y=Y, squared=squared)
+    nan_distance = nan_euclidean_distances(X, Y=Y, squared=squared)
+    assert_allclose(normal_distance, nan_distance)
+
+
+@pytest.mark.parametrize(
+    "X", [np.array([[np.inf, 0]]), np.array([[0, -np.inf]])])
+@pytest.mark.parametrize(
+    "Y", [np.array([[np.inf, 0]]), np.array([[0, -np.inf]]), None])
+def test_nan_euclidean_distances_infinite_values(X, Y):
+
+    with pytest.raises(ValueError) as excinfo:
+        nan_euclidean_distances(X, Y=Y)
+
+    exp_msg = ("Input contains infinity or a value too large for "
+               "dtype('float64').")
+    assert exp_msg == str(excinfo.value)
+
+
+@pytest.mark.parametrize("X, X_diag, missing_value", [
+    (np.array([[0, 1], [1, 0]]), np.sqrt(2), np.nan),
+    (np.array([[0, 1], [1, np.nan]]), np.sqrt(2), np.nan),
+    (np.array([[np.nan, 1], [1, np.nan]]), np.nan, np.nan),
+    (np.array([[np.nan, 1], [np.nan, 0]]), np.sqrt(2), np.nan),
+    (np.array([[0, np.nan], [1, np.nan]]), np.sqrt(2), np.nan),
+    (np.array([[0, 1], [1, 0]]), np.sqrt(2), -1),
+    (np.array([[0, 1], [1, -1]]), np.sqrt(2), -1),
+    (np.array([[-1, 1], [1, -1]]), np.nan, -1),
+    (np.array([[-1, 1], [-1, 0]]), np.sqrt(2), -1),
+    (np.array([[0, -1], [1, -1]]), np.sqrt(2), -1)
+])
+def test_nan_euclidean_distances_2x2(X, X_diag, missing_value):
+
+    exp_dist = np.array([[0., X_diag], [X_diag, 0]])
+
+    dist = nan_euclidean_distances(X, missing_values=missing_value)
+    assert_allclose(exp_dist, dist)
+
+    dist_sq = nan_euclidean_distances(
+        X, squared=True, missing_values=missing_value)
+    assert_allclose(exp_dist**2, dist_sq)
+
+    dist_two = nan_euclidean_distances(X, X, missing_values=missing_value)
+    assert_allclose(exp_dist, dist_two)
+
+    dist_two_copy = nan_euclidean_distances(
+        X, X.copy(), missing_values=missing_value)
+    assert_allclose(exp_dist, dist_two_copy)
+
+
+@pytest.mark.parametrize("missing_value", [np.nan, -1])
+def test_nan_euclidean_distances_complete_nan(missing_value):
+    X = np.array([[missing_value, missing_value], [0, 1]])
+
+    exp_dist = np.array([[np.nan, np.nan], [np.nan, 0]])
+
+    dist = nan_euclidean_distances(X, missing_values=missing_value)
+    assert_allclose(exp_dist, dist)
+
+    dist = nan_euclidean_distances(
+            X, X.copy(), missing_values=missing_value)
+    assert_allclose(exp_dist, dist)
+
+
+@pytest.mark.parametrize("missing_value", [np.nan, -1])
+def test_nan_euclidean_distances_not_trival(missing_value):
+    X = np.array([[1., missing_value, 3., 4., 2.],
+                  [missing_value, 4., 6., 1., missing_value],
+                  [3., missing_value, missing_value, missing_value, 1.]])
+
+    Y = np.array([[missing_value, 7., 7., missing_value, 2.],
+                  [missing_value, missing_value, 5., 4., 7.],
+                  [missing_value, missing_value, missing_value, 4., 5.]])
+
+    # Check for symmetry
+    D1 = nan_euclidean_distances(X, Y,  missing_values=missing_value)
+    D2 = nan_euclidean_distances(Y, X, missing_values=missing_value)
+
+    assert_almost_equal(D1, D2.T)
+
+    # Check with explicit formula and squared=True
+    assert_allclose(
+        nan_euclidean_distances(
+            X[:1], Y[:1], squared=True, missing_values=missing_value),
+        [[5.0 / 2.0 * ((7 - 3)**2 + (2 - 2)**2)]])
+
+    # Check with explicit formula and squared=False
+    assert_allclose(
+        nan_euclidean_distances(
+            X[1:2], Y[1:2], squared=False, missing_values=missing_value),
+        [[np.sqrt(5.0 / 2.0 * ((6 - 5)**2 + (1 - 4)**2))]])
+
+    # Check when Y = X is explicitly passed
+    D3 = nan_euclidean_distances(X, missing_values=missing_value)
+    D4 = nan_euclidean_distances(X, X, missing_values=missing_value)
+    D5 = nan_euclidean_distances(X, X.copy(), missing_values=missing_value)
+    assert_allclose(D3, D4)
+    assert_allclose(D4, D5)
+
+    # Check copy = True against copy = False
+    D6 = nan_euclidean_distances(X, Y, copy=True)
+    D7 = nan_euclidean_distances(X, Y, copy=False)
+    assert_allclose(D6, D7)
+
+
+@pytest.mark.parametrize("missing_value", [np.nan, -1])
+def test_nan_euclidean_distances_one_feature_match_positive(missing_value):
+    # First feature is the only feature that is non-nan and in both
+    # samples. The result of `nan_euclidean_distances` with squared=True
+    # should be non-negative. The non-squared version should all be close to 0.
+    X = np.array([[-122.27, 648., missing_value, 37.85],
+                  [-122.27, missing_value, 2.34701493, missing_value]])
+
+    dist_squared = nan_euclidean_distances(X, missing_values=missing_value,
+                                           squared=True)
+    assert np.all(dist_squared >= 0)
+
+    dist = nan_euclidean_distances(X, missing_values=missing_value,
+                                   squared=False)
+    assert_allclose(dist, 0.0)
 
 
 def test_cosine_distances():
@@ -606,6 +931,29 @@ def test_cosine_distances():
     assert np.all(D <= 2.)
 
 
+def test_haversine_distances():
+    # Check haversine distance with distances computation
+    def slow_haversine_distances(x, y):
+        diff_lat = y[0] - x[0]
+        diff_lon = y[1] - x[1]
+        a = np.sin(diff_lat / 2) ** 2 + (
+            np.cos(x[0]) * np.cos(y[0]) * np.sin(diff_lon/2) ** 2
+        )
+        c = 2 * np.arcsin(np.sqrt(a))
+        return c
+    rng = np.random.RandomState(0)
+    X = rng.random_sample((5, 2))
+    Y = rng.random_sample((10, 2))
+    D1 = np.array([[slow_haversine_distances(x, y) for y in Y] for x in X])
+    D2 = haversine_distances(X, Y)
+    assert_array_almost_equal(D1, D2)
+    # Test haversine distance does not accept X where n_feature != 2
+    X = rng.random_sample((10, 3))
+    err_msg = "Haversine distance only valid in 2 dimensions"
+    with pytest.raises(ValueError, match=err_msg):
+        haversine_distances(X)
+
+
 # Paired distances
 
 def test_paired_euclidean_distances():
@@ -631,7 +979,7 @@ def test_chi_square_kernel():
     K_add = additive_chi2_kernel(X, Y)
     gamma = 0.1
     K = chi2_kernel(X, Y, gamma=gamma)
-    assert_equal(K.dtype, np.float)
+    assert K.dtype == np.float
     for i, x in enumerate(X):
         for j, y in enumerate(Y):
             chi2 = -np.sum((x - y) ** 2 / (x + y))
@@ -649,34 +997,39 @@ def test_chi_square_kernel():
     X = rng.random_sample((5, 4)).astype(np.float32)
     Y = rng.random_sample((10, 4)).astype(np.float32)
     K = chi2_kernel(X, Y)
-    assert_equal(K.dtype, np.float32)
+    assert K.dtype == np.float32
 
     # check integer type gets converted,
     # check that zeros are handled
     X = rng.random_sample((10, 4)).astype(np.int32)
     K = chi2_kernel(X, X)
     assert np.isfinite(K).all()
-    assert_equal(K.dtype, np.float)
+    assert K.dtype == np.float
 
     # check that kernel of similar things is greater than dissimilar ones
     X = [[.3, .7], [1., 0]]
     Y = [[0, 1], [.9, .1]]
     K = chi2_kernel(X, Y)
-    assert_greater(K[0, 0], K[0, 1])
-    assert_greater(K[1, 1], K[1, 0])
+    assert K[0, 0] > K[0, 1]
+    assert K[1, 1] > K[1, 0]
 
     # test negative input
-    assert_raises(ValueError, chi2_kernel, [[0, -1]])
-    assert_raises(ValueError, chi2_kernel, [[0, -1]], [[-1, -1]])
-    assert_raises(ValueError, chi2_kernel, [[0, 1]], [[-1, -1]])
+    with pytest.raises(ValueError):
+        chi2_kernel([[0, -1]])
+    with pytest.raises(ValueError):
+        chi2_kernel([[0, -1]], [[-1, -1]])
+    with pytest.raises(ValueError):
+        chi2_kernel([[0, 1]], [[-1, -1]])
 
     # different n_features in X and Y
-    assert_raises(ValueError, chi2_kernel, [[0, 1]], [[.2, .2, .6]])
+    with pytest.raises(ValueError):
+        chi2_kernel([[0, 1]], [[.2, .2, .6]])
 
     # sparse matrices
-    assert_raises(ValueError, chi2_kernel, csr_matrix(X), csr_matrix(Y))
-    assert_raises(ValueError, additive_chi2_kernel,
-                  csr_matrix(X), csr_matrix(Y))
+    with pytest.raises(ValueError):
+        chi2_kernel(csr_matrix(X), csr_matrix(Y))
+    with pytest.raises(ValueError):
+        additive_chi2_kernel(csr_matrix(X), csr_matrix(Y))
 
 
 @pytest.mark.parametrize(
@@ -806,10 +1159,12 @@ def test_check_different_dimensions():
     # Ensure an error is raised if the dimensions are different.
     XA = np.resize(np.arange(45), (5, 9))
     XB = np.resize(np.arange(32), (4, 8))
-    assert_raises(ValueError, check_pairwise_arrays, XA, XB)
+    with pytest.raises(ValueError):
+        check_pairwise_arrays(XA, XB)
 
     XB = np.resize(np.arange(4 * 9), (4, 9))
-    assert_raises(ValueError, check_paired_arrays, XA, XB)
+    with pytest.raises(ValueError):
+        check_paired_arrays(XA, XB)
 
 
 def test_check_invalid_dimensions():
@@ -818,10 +1173,12 @@ def test_check_invalid_dimensions():
     # converted to 2D anyways
     XA = np.arange(45).reshape(9, 5)
     XB = np.arange(32).reshape(4, 8)
-    assert_raises(ValueError, check_pairwise_arrays, XA, XB)
+    with pytest.raises(ValueError):
+        check_pairwise_arrays(XA, XB)
     XA = np.arange(45).reshape(9, 5)
     XB = np.arange(32).reshape(4, 8)
-    assert_raises(ValueError, check_pairwise_arrays, XA, XB)
+    with pytest.raises(ValueError):
+        check_pairwise_arrays(XA, XB)
 
 
 def test_check_sparse_arrays():
@@ -835,15 +1192,15 @@ def test_check_sparse_arrays():
     # compare their difference because testing csr matrices for
     # equality with '==' does not work as expected.
     assert issparse(XA_checked)
-    assert_equal(abs(XA_sparse - XA_checked).sum(), 0)
+    assert abs(XA_sparse - XA_checked).sum() == 0
     assert issparse(XB_checked)
-    assert_equal(abs(XB_sparse - XB_checked).sum(), 0)
+    assert abs(XB_sparse - XB_checked).sum() == 0
 
     XA_checked, XA_2_checked = check_pairwise_arrays(XA_sparse, XA_sparse)
     assert issparse(XA_checked)
-    assert_equal(abs(XA_sparse - XA_checked).sum(), 0)
+    assert abs(XA_sparse - XA_checked).sum() == 0
     assert issparse(XA_2_checked)
-    assert_equal(abs(XA_2_checked - XA_checked).sum(), 0)
+    assert abs(XA_2_checked - XA_checked).sum() == 0
 
 
 def tuplify(X):
@@ -875,21 +1232,57 @@ def test_check_preserve_type():
     XB = np.resize(np.arange(40), (5, 8)).astype(np.float32)
 
     XA_checked, XB_checked = check_pairwise_arrays(XA, None)
-    assert_equal(XA_checked.dtype, np.float32)
+    assert XA_checked.dtype == np.float32
 
     # both float32
     XA_checked, XB_checked = check_pairwise_arrays(XA, XB)
-    assert_equal(XA_checked.dtype, np.float32)
-    assert_equal(XB_checked.dtype, np.float32)
+    assert XA_checked.dtype == np.float32
+    assert XB_checked.dtype == np.float32
 
     # mismatched A
     XA_checked, XB_checked = check_pairwise_arrays(XA.astype(np.float),
                                                    XB)
-    assert_equal(XA_checked.dtype, np.float)
-    assert_equal(XB_checked.dtype, np.float)
+    assert XA_checked.dtype == np.float
+    assert XB_checked.dtype == np.float
 
     # mismatched B
     XA_checked, XB_checked = check_pairwise_arrays(XA,
                                                    XB.astype(np.float))
-    assert_equal(XA_checked.dtype, np.float)
-    assert_equal(XB_checked.dtype, np.float)
+    assert XA_checked.dtype == np.float
+    assert XB_checked.dtype == np.float
+
+
+@pytest.mark.parametrize("n_jobs", [1, 2])
+@pytest.mark.parametrize("metric", ["seuclidean", "mahalanobis"])
+@pytest.mark.parametrize("dist_function",
+                         [pairwise_distances, pairwise_distances_chunked])
+@pytest.mark.parametrize("y_is_x", [True, False], ids=["Y is X", "Y is not X"])
+def test_pairwise_distances_data_derived_params(n_jobs, metric, dist_function,
+                                                y_is_x):
+    # check that pairwise_distances give the same result in sequential and
+    # parallel, when metric has data-derived parameters.
+    with config_context(working_memory=0.1):  # to have more than 1 chunk
+        rng = np.random.RandomState(0)
+        X = rng.random_sample((100, 10))
+
+        if y_is_x:
+            Y = X
+            expected_dist_default_params = squareform(pdist(X, metric=metric))
+            if metric == "seuclidean":
+                params = {'V': np.var(X, axis=0, ddof=1)}
+            else:
+                params = {'VI': np.linalg.inv(np.cov(X.T)).T}
+        else:
+            Y = rng.random_sample((100, 10))
+            expected_dist_default_params = cdist(X, Y, metric=metric)
+            if metric == "seuclidean":
+                params = {'V': np.var(np.vstack([X, Y]), axis=0, ddof=1)}
+            else:
+                params = {'VI': np.linalg.inv(np.cov(np.vstack([X, Y]).T)).T}
+
+        expected_dist_explicit_params = cdist(X, Y, metric=metric, **params)
+        dist = np.vstack(tuple(dist_function(X, Y,
+                                             metric=metric, n_jobs=n_jobs)))
+
+        assert_allclose(dist, expected_dist_explicit_params)
+        assert_allclose(dist, expected_dist_default_params)
