@@ -3,30 +3,38 @@ import sys
 
 import numpy as np
 import scipy.sparse as sp
+import joblib
 
-from sklearn.externals.six.moves import cStringIO as StringIO
+from io import StringIO
 
 from sklearn.base import BaseEstimator, ClassifierMixin
 from sklearn.utils import deprecated
-from sklearn.utils import _joblib
-from sklearn.utils.testing import (assert_raises_regex, assert_equal,
-                                   ignore_warnings, assert_warns)
-from sklearn.utils.estimator_checks import check_estimator
+from sklearn.utils._testing import (assert_raises_regex,
+                                   ignore_warnings,
+                                   assert_warns, assert_raises,
+                                   SkipTest)
+from sklearn.utils.estimator_checks import check_estimator, _NotAnArray
+from sklearn.utils.estimator_checks \
+    import check_class_weight_balanced_linear_classifier
 from sklearn.utils.estimator_checks import set_random_state
-from sklearn.utils.estimator_checks import set_checking_parameters
+from sklearn.utils.estimator_checks import _set_checking_parameters
 from sklearn.utils.estimator_checks import check_estimators_unfitted
 from sklearn.utils.estimator_checks import check_fit_score_takes_y
 from sklearn.utils.estimator_checks import check_no_attributes_set_in_init
-from sklearn.ensemble import AdaBoostClassifier, RandomForestClassifier
+from sklearn.utils.validation import check_is_fitted
+from sklearn.utils.estimator_checks import check_outlier_corruption
+from sklearn.utils.fixes import _parse_version
+from sklearn.ensemble import RandomForestClassifier
 from sklearn.linear_model import LinearRegression, SGDClassifier
 from sklearn.mixture import GaussianMixture
 from sklearn.cluster import MiniBatchKMeans
 from sklearn.decomposition import NMF
-from sklearn.linear_model import MultiTaskElasticNet
+from sklearn.linear_model import MultiTaskElasticNet, LogisticRegression
 from sklearn.svm import SVC
 from sklearn.neighbors import KNeighborsRegressor
-from sklearn.utils.validation import (check_X_y, check_array,
-                                      LARGE_SPARSE_SUPPORTED)
+from sklearn.tree import DecisionTreeClassifier
+from sklearn.utils.validation import check_X_y, check_array
+from sklearn.utils import all_estimators
 
 
 class CorrectNotFittedError(ValueError):
@@ -37,7 +45,7 @@ class CorrectNotFittedError(ValueError):
     """
 
 
-class BaseBadClassifier(BaseEstimator, ClassifierMixin):
+class BaseBadClassifier(ClassifierMixin, BaseEstimator):
     def fit(self, X, y):
         return self
 
@@ -96,7 +104,7 @@ class RaisesErrorInSetParams(BaseEstimator):
             if p < 0:
                 raise ValueError("p can't be less than 0")
             self.p = p
-        return super(RaisesErrorInSetParams, self).set_params(**kwargs)
+        return super().set_params(**kwargs)
 
     def fit(self, X, y=None):
         X, y = check_X_y(X, y)
@@ -113,8 +121,7 @@ class ModifiesValueInsteadOfRaisingError(BaseEstimator):
             if p < 0:
                 p = 0
             self.p = p
-        return super(ModifiesValueInsteadOfRaisingError,
-                     self).set_params(**kwargs)
+        return super().set_params(**kwargs)
 
     def fit(self, X, y=None):
         X, y = check_X_y(X, y)
@@ -133,8 +140,7 @@ class ModifiesAnotherValue(BaseEstimator):
             if a is None:
                 kwargs.pop('b')
                 self.b = 'method2'
-        return super(ModifiesAnotherValue,
-                     self).set_params(**kwargs)
+        return super().set_params(**kwargs)
 
     def fit(self, X, y=None):
         X, y = check_X_y(X, y)
@@ -166,8 +172,7 @@ class CorrectNotFittedErrorClassifier(BaseBadClassifier):
         return self
 
     def predict(self, X):
-        if not hasattr(self, 'coef_'):
-            raise CorrectNotFittedError("estimator is not fitted yet")
+        check_is_fitted(self)
         X = check_array(X)
         return np.ones(X.shape[0])
 
@@ -191,6 +196,28 @@ class NoSampleWeightPandasSeriesType(BaseEstimator):
         return np.ones(X.shape[0])
 
 
+class BadBalancedWeightsClassifier(BaseBadClassifier):
+    def __init__(self, class_weight=None):
+        self.class_weight = class_weight
+
+    def fit(self, X, y):
+        from sklearn.preprocessing import LabelEncoder
+        from sklearn.utils import compute_class_weight
+
+        label_encoder = LabelEncoder().fit(y)
+        classes = label_encoder.classes_
+        class_weight = compute_class_weight(self.class_weight, classes, y)
+
+        # Intentionally modify the balanced class_weight
+        # to simulate a bug and raise an exception
+        if self.class_weight == "balanced":
+            class_weight += 1.
+
+        # Simply assigning coef_ to the class_weight
+        self.coef_ = class_weight
+        return self
+
+
 class BadTransformerWithoutMixin(BaseEstimator):
     def fit(self, X, y=None):
         X = check_array(X)
@@ -253,6 +280,44 @@ class SparseTransformer(BaseEstimator):
         return sp.csr_matrix(X)
 
 
+class UntaggedBinaryClassifier(DecisionTreeClassifier):
+    # Toy classifier that only supports binary classification, will fail tests.
+    def fit(self, X, y, sample_weight=None):
+        super().fit(X, y, sample_weight)
+        if np.all(self.n_classes_ > 2):
+            raise ValueError('Only 2 classes are supported')
+        return self
+
+
+class TaggedBinaryClassifier(UntaggedBinaryClassifier):
+    # Toy classifier that only supports binary classification.
+    def _more_tags(self):
+        return {'binary_only': True}
+
+
+class RequiresPositiveYRegressor(LinearRegression):
+
+    def fit(self, X, y):
+        X, y = check_X_y(X, y, multi_output=True)
+        if (y <= 0).any():
+            raise ValueError('negative y values not supported!')
+        return super().fit(X, y)
+
+    def _more_tags(self):
+        return {"requires_positive_y": True}
+
+
+def test_not_an_array_array_function():
+    np_version = _parse_version(np.__version__)
+    if np_version < (1, 17):
+        raise SkipTest("array_function protocol not supported in numpy <1.17")
+    not_array = _NotAnArray(np.ones(10))
+    msg = "Don't want to call array_function sum!"
+    assert_raises_regex(TypeError, msg, np.sum, not_array)
+    # always returns True
+    assert np.may_share_memory(not_array, None)
+
+
 def test_check_fit_score_takes_y_works_on_deprecated_fit():
     # Tests that check_fit_score_takes_y works on a class with
     # a deprecated fit method
@@ -286,7 +351,7 @@ def test_check_estimator():
     assert_raises_regex(AttributeError, msg, check_estimator, BaseEstimator)
     assert_raises_regex(AttributeError, msg, check_estimator, BaseEstimator())
     # check that fit does input validation
-    msg = "TypeError not raised"
+    msg = "ValueError not raised"
     assert_raises_regex(AssertionError, msg, check_estimator,
                         BaseBadClassifier)
     assert_raises_regex(AssertionError, msg, check_estimator,
@@ -350,20 +415,40 @@ def test_check_estimator():
     # Large indices test on bad estimator
     msg = ('Estimator LargeSparseNotSupportedClassifier doesn\'t seem to '
            r'support \S{3}_64 matrix, and is not failing gracefully.*')
-    # only supported by scipy version more than 0.14.0
-    if LARGE_SPARSE_SUPPORTED:
-        assert_raises_regex(AssertionError, msg, check_estimator,
-                            LargeSparseNotSupportedClassifier)
+    assert_raises_regex(AssertionError, msg, check_estimator,
+                        LargeSparseNotSupportedClassifier)
+
+    # does error on binary_only untagged estimator
+    msg = 'Only 2 classes are supported'
+    assert_raises_regex(ValueError, msg, check_estimator,
+                        UntaggedBinaryClassifier)
 
     # non-regression test for estimators transforming to sparse data
     check_estimator(SparseTransformer())
 
     # doesn't error on actual estimator
-    check_estimator(AdaBoostClassifier)
-    check_estimator(AdaBoostClassifier())
+    check_estimator(LogisticRegression)
+    check_estimator(LogisticRegression(C=0.01))
     check_estimator(MultiTaskElasticNet)
     check_estimator(MultiTaskElasticNet())
 
+    # doesn't error on binary_only tagged estimator
+    check_estimator(TaggedBinaryClassifier)
+
+    # Check regressor with requires_positive_y estimator tag
+    msg = 'negative y values not supported!'
+    assert_raises_regex(ValueError, msg, check_estimator,
+                        RequiresPositiveYRegressor)
+
+
+def test_check_outlier_corruption():
+    # should raise AssertionError
+    decision = np.array([0., 1., 1.5, 2.])
+    assert_raises(AssertionError, check_outlier_corruption, 1, 2, decision)
+    # should pass
+    decision = np.array([0., 1., 1., 2.])
+    check_outlier_corruption(1, 2, decision)
+
 
 def test_check_estimator_transformer_no_mixin():
     # check that TransformerMixin is not required for transformer tests to run
@@ -379,32 +464,32 @@ def test_check_estimator_clones():
     for Estimator in [GaussianMixture, LinearRegression,
                       RandomForestClassifier, NMF, SGDClassifier,
                       MiniBatchKMeans]:
-        with ignore_warnings(category=(FutureWarning, DeprecationWarning)):
+        with ignore_warnings(category=FutureWarning):
             # when 'est = SGDClassifier()'
             est = Estimator()
-            set_checking_parameters(est)
+            _set_checking_parameters(est)
             set_random_state(est)
             # without fitting
-            old_hash = _joblib.hash(est)
+            old_hash = joblib.hash(est)
             check_estimator(est)
-        assert_equal(old_hash, _joblib.hash(est))
+        assert old_hash == joblib.hash(est)
 
-        with ignore_warnings(category=(FutureWarning, DeprecationWarning)):
+        with ignore_warnings(category=FutureWarning):
             # when 'est = SGDClassifier()'
             est = Estimator()
-            set_checking_parameters(est)
+            _set_checking_parameters(est)
             set_random_state(est)
             # with fitting
             est.fit(iris.data + 10, iris.target)
-            old_hash = _joblib.hash(est)
+            old_hash = joblib.hash(est)
             check_estimator(est)
-        assert_equal(old_hash, _joblib.hash(est))
+        assert old_hash == joblib.hash(est)
 
 
 def test_check_estimators_unfitted():
     # check that a ValueError/AttributeError is raised when calling predict
     # on an unfitted estimator
-    msg = "AttributeError or ValueError not raised by predict"
+    msg = "NotFittedError not raised by predict"
     assert_raises_regex(AssertionError, msg, check_estimators_unfitted,
                         "estimator", NoSparseClassifier())
 
@@ -414,11 +499,11 @@ def test_check_estimators_unfitted():
 
 
 def test_check_no_attributes_set_in_init():
-    class NonConformantEstimatorPrivateSet(object):
+    class NonConformantEstimatorPrivateSet(BaseEstimator):
         def __init__(self):
             self.you_should_not_set_this_ = None
 
-    class NonConformantEstimatorNoParamSet(object):
+    class NonConformantEstimatorNoParamSet(BaseEstimator):
         def __init__(self, you_should_set_this_=None):
             pass
 
@@ -441,7 +526,7 @@ def test_check_no_attributes_set_in_init():
 
 def test_check_estimator_pairwise():
     # check that check_estimator() works on estimator with _pairwise
-    # kernel or  metric
+    # kernel or metric
 
     # test precomputed kernel
     est = SVC(kernel='precomputed')
@@ -452,6 +537,19 @@ def test_check_estimator_pairwise():
     check_estimator(est)
 
 
+def test_check_estimator_required_parameters_skip():
+    class MyEstimator(BaseEstimator):
+        _required_parameters = ["special_parameter"]
+
+        def __init__(self, special_parameter):
+            self.special_parameter = special_parameter
+
+    assert_raises_regex(SkipTest, r"Can't instantiate estimator MyEstimator "
+                                  r"which requires parameters "
+                                  r"\['special_parameter'\]",
+                                  check_estimator, MyEstimator)
+
+
 def run_tests_without_pytest():
     """Runs the tests in this file without using pytest.
     """
@@ -465,6 +563,24 @@ def run_tests_without_pytest():
     runner.run(suite)
 
 
+def test_check_class_weight_balanced_linear_classifier():
+    # check that ill-computed balanced weights raises an exception
+    assert_raises_regex(AssertionError,
+                        "Classifier estimator_name is not computing"
+                        " class_weight=balanced properly.",
+                        check_class_weight_balanced_linear_classifier,
+                        'estimator_name',
+                        BadBalancedWeightsClassifier)
+
+
+def test_all_estimators_all_public():
+    # all_estimator should not fail when pytest is not installed and return
+    # only public estimators
+    estimators = all_estimators()
+    for est in estimators:
+        assert not est.__class__.__name__.startswith("_")
+
+
 if __name__ == '__main__':
     # This module is run as a script to check that we have no dependency on
     # pytest for estimator checks.
